<?php

namespace Dedi\SyliusCustomerPricingPlugin\Controller\Trait;

use Dedi\SyliusCustomerPricingPlugin\Entity\ChannelPricing\TypeAwareInterface;
use Dedi\SyliusCustomerPricingPlugin\Enum\ChannelPricingType;
use LogicException;
use Sylius\Component\Core\Model\ChannelPricingInterface;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;

trait ChannelPricingControllerTrait
{
    public function redirectAction(
        Request $request
    ): Response
    {
        $configuration = $this->requestConfigurationFactory->create($this->metadata, $request);

        /** @var ChannelPricingInterface $channelPricing */
        $channelPricing = $this->findOr404($configuration);

        if (!$channelPricing instanceof TypeAwareInterface) {
            throw new LogicException('The channelPricing should implement TypeAwareInterface.');
        }

        $type = $channelPricing->getType();

        // guess the route from the channelPricing type
        $redirectConfig = $configuration->getParameters()->get('redirect');
        if (!is_array($redirectConfig) || !array_key_exists('routes', $redirectConfig)) {
            throw new LogicException('This route should be configured with a redirect and a list of routes.');
        }
        if (!is_array($redirectConfig['routes']) || !array_key_exists($type, $redirectConfig['routes'])) {
            throw new LogicException(sprintf(
                'routes should be an array and a route name should be configured as the value of the %s key',
                $type,
            ));
        }

        $redirectConfig = array_merge($redirectConfig, [
            'route' => $redirectConfig['routes'][$type],
        ]);
        $configuration->getParameters()->set('redirect', $redirectConfig);

        return $this->redirectHandler->redirectToResource($configuration, $channelPricing);
    }
}
