<?php

declare(strict_types=1);

namespace Dedi\SyliusCustomerPricingPlugin\Loader;

use Dedi\SyliusCustomerPricingPlugin\Entity\ChannelPricing\ChannelPricingSubjectInterface;
use Dedi\SyliusCustomerPricingPlugin\Loader\ValueObject\ChannelPricing;
use Dedi\SyliusCustomerPricingPlugin\Repository\ChannelPricingRepositoryInterface;
use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\Common\Collections\Collection;

class CompositeChannelPricingLoader implements ChannelPricingLoaderInterface
{
    public function __construct(
        private iterable $loaders,
        private ChannelPricingRepositoryInterface $channelPricingRepository,
    ) {
        $this->loaders = array_reverse(iterator_to_array($this->loaders));
    }

    public function load(ChannelPricingSubjectInterface $object): Collection
    {
        $channelPricings = [];
        /** @var ChannelPricingLoaderInterface $loader */
        foreach ($this->loaders as $loader) {
            if ($loader->supports($object)) {
                $channelPricings[] = $loader->load($object)->toArray();
            }
        }
        $channelPricings = array_merge(...$channelPricings);

        return new ArrayCollection(array_map(function (ChannelPricing $channelPricing) {
            return $this->channelPricingRepository->find($channelPricing->getId());
        }, $channelPricings));
    }

    public function supports(ChannelPricingSubjectInterface $object): bool
    {
        return true;
    }
}
