<?php

declare(strict_types=1);

namespace Dedi\SyliusCustomerPricingPlugin\Menu;

use Dedi\SyliusCustomerPricingPlugin\Entity\Customer\OrganizationInterface;
use Dedi\SyliusCustomerPricingPlugin\Event\OrganizationShowMenuBuilderEvent;
use Knp\Menu\FactoryInterface;
use Knp\Menu\ItemInterface;
use Symfony\Component\EventDispatcher\EventDispatcherInterface;

final class OrganizationShowMenuBuilder
{
    public const EVENT_NAME = 'dedi_sylius_customer_pricing_plugin.menu.admin.organization.show';

    public function __construct(private FactoryInterface $factory, private EventDispatcherInterface $eventDispatcher)
    {
    }

    public function createMenu(array $options): ItemInterface
    {
        $menu = $this->factory->createItem('root');

        if (!isset($options['organization'])) {
            return $menu;
        }

        $organization = $options['organization'];
        $this->addChildren($menu, $organization);

        $this->eventDispatcher->dispatch(
            new OrganizationShowMenuBuilderEvent($this->factory, $menu, $organization),
            self::EVENT_NAME,
        );

        return $menu;
    }

    private function addChildren(ItemInterface $menu, OrganizationInterface $organization): void
    {
        $menu->setExtra('column_id', 'actions');

        $menu
            ->addChild('update', [
                'route' => 'dedi_sylius_customer_pricing_plugin_admin_organization_update',
                'routeParameters' => ['id' => $organization->getId()],
            ])
            ->setAttribute('type', 'edit')
            ->setLabel('sylius.ui.edit');

        $menu
            ->addChild('customer_index', [
                'route' => 'dedi_sylius_customer_pricing_plugin_admin_organization_customer_index',
                'routeParameters' => ['id' => $organization->getId()],
            ])
            ->setAttribute('type', 'show')
            ->setLabel('dedi_sylius_customer_pricing_plugin.ui.show_customers');

        $menu
            ->addChild('order_index', [
                'route' => 'dedi_sylius_customer_pricing_plugin_admin_organization_order_index',
                'routeParameters' => ['id' => $organization->getId()],
            ])
            ->setAttribute('type', 'show')
            ->setLabel('dedi_sylius_customer_pricing_plugin.ui.show_orders');

        $menu
            ->addChild('organization_delete', [
                'route' => 'dedi_sylius_customer_pricing_plugin_admin_organization_delete',
                'routeParameters' => ['id' => $organization->getId()],
            ])
            ->setAttribute('type', 'delete')
            ->setAttribute('resource_id', $organization->getId())
            ->setLabel('sylius.ui.delete');
    }
}
