<?php

declare(strict_types=1);

namespace Dedi\SyliusCustomerPricingPlugin\Validator;

use Dedi\SyliusCustomerPricingPlugin\Entity\ChannelPricing\Default\DefaultChannelPricingInterface;
use Dedi\SyliusCustomerPricingPlugin\Entity\ChannelPricing\TypeAwareInterface;
use Dedi\SyliusCustomerPricingPlugin\Enum\ChannelPricingType;
use Dedi\SyliusCustomerPricingPlugin\Repository\ChannelPricingRepositoryInterface;
use Dedi\SyliusCustomerPricingPlugin\Validator\Constraints\UniqueDefaultChannelPricing;
use Sylius\Component\Core\Model\ChannelPricingInterface;
use Symfony\Component\Validator\Constraint;
use Symfony\Component\Validator\ConstraintValidator;
use Symfony\Component\Validator\Exception\UnexpectedValueException;

class UniqueDefaultChannelPricingValidator extends ConstraintValidator
{
    public function __construct(
        private readonly ChannelPricingRepositoryInterface $channelPricingRepository
    )
    {

    }

    public function validate(mixed $value, Constraint $constraint)
    {
        if (!$value instanceof ChannelPricingInterface) {
            throw new UnexpectedValueException($value, ChannelPricingInterface::class);
        }

        if (!$constraint instanceof UniqueDefaultChannelPricing) {
            throw new UnexpectedValueException($constraint, UniqueDefaultChannelPricing::class);
        }

        if (!$value instanceof TypeAwareInterface) {
            throw new UnexpectedValueException($value, TypeAwareInterface::class);
        }

        if (ChannelPricingType::DEFAULT->value !== $value->getType()) {
            return;
        }

        if (null === $value->getProductVariant()) {
            return;
        }

        $existing = $this->channelPricingRepository->findAllDefaultsForProductVariantAndChannel($value->getChannelCode(), $value->getProductVariant());

        $existing = array_filter($existing, static fn(ChannelPricingInterface $item) => $item->getId() !== $value->getId());

        if (count($existing) > 0) {
            $this->context->buildViolation($constraint->defaultChannelPricingAlreadyExistsMessage)
                ->addViolation();
        }
    }
}
