<?php

declare(strict_types=1);

namespace Dedi\SyliusCustomerPricingPlugin\Validator;

use Dedi\SyliusCustomerPricingPlugin\Entity\ChannelPricing\Organization\OrganizationChannelPricingInterface;
use Dedi\SyliusCustomerPricingPlugin\Repository\ChannelPricingRepositoryInterface;
use Dedi\SyliusCustomerPricingPlugin\Validator\Constraints\UniqueOrganizationChannelPricing;
use Symfony\Component\Validator\Constraint;
use Symfony\Component\Validator\ConstraintValidator;
use Symfony\Component\Validator\Exception\UnexpectedValueException;

class UniqueOrganizationChannelPricingValidator extends ConstraintValidator
{
    public function __construct(
        private readonly ChannelPricingRepositoryInterface $channelPricingRepository
    )
    {

    }

    public function validate(mixed $value, Constraint $constraint)
    {
        if (!$value instanceof OrganizationChannelPricingInterface) {
            throw new UnexpectedValueException($value, OrganizationChannelPricingInterface::class);
        }

        if (!$constraint instanceof UniqueOrganizationChannelPricing) {
            throw new UnexpectedValueException($constraint, UniqueOrganizationChannelPricing::class);
        }

        if (null === $value->getProductVariant()
            || null === $value->getOrganization()
        ) {
            return;
        }

        $existing = $this->channelPricingRepository->findAllForOrganizationForProductVariantAndChannel($value->getChannelCode(), $value->getProductVariant(), $value->getOrganization());

        $existing = array_filter($existing, static fn(OrganizationChannelPricingInterface $item) => $item->getId() !== $value->getId());

        if (count($existing) > 0) {
            $this->context->buildViolation($constraint->organizationChannelPricingAlreadyExistsMessage)
                ->addViolation();
        }
    }
}
